import {
  Button,
  Paper,
  TextField,
  Select,
  MenuItem,
  FormControl,
  InputLabel,
  // Typography,
  Box,
} from "@mui/material";
import { LanguageSwitcher } from "@widgets";
import { observer } from "mobx-react-lite";
import { ArrowLeft, Save } from "lucide-react";
import { useEffect, useState } from "react";
import { useNavigate, useParams } from "react-router-dom";

import { carrierStore } from "../../../shared/store/CarrierStore";
import { articlesStore } from "../../../shared/store/ArticlesStore";
import { routeStore } from "../../../shared/store/RouteStore";
import { toast } from "react-toastify";
import { languageStore, stationsStore } from "@shared";
import { LinkedItems } from "../LinekedStations";

export const RouteEditPage = observer(() => {
  const navigate = useNavigate();
  const { id } = useParams();
  const { editRouteData } = routeStore;
  const [isLoading, setIsLoading] = useState(false);
  const { language } = languageStore;
  const [coordinates, setCoordinates] = useState<string>("");

  useEffect(() => {
    const fetchData = async () => {
      const response = await routeStore.getRoute(Number(id));
      routeStore.setEditRouteData(response);
      carrierStore.getCarriers(language);
      stationsStore.getStations();
      articlesStore.getArticleList();
    };
    fetchData();
  }, [id, language]);

  useEffect(() => {
    if (editRouteData.path && editRouteData.path.length > 0) {
      const formattedPath = editRouteData.path
        .map((coords) => coords.join(" "))
        .join("\n");
      setCoordinates(formattedPath);
    }
  }, [editRouteData.path]);

  const handleSave = async () => {
    setIsLoading(true);
    await routeStore.editRoute(Number(id));
    toast.success("Маршрут успешно сохранен");
    setIsLoading(false);
  };

  const validateCoordinates = (value: string) => {
    try {
      const lines = value.trim().split("\n");
      const coordinates = lines.map((line) => {
        const [lat, lon] = line
          .trim()
          .split(/[\s,]+/)
          .map(Number);
        return [lat, lon];
      });

      if (coordinates.length === 0) {
        return "Введите хотя бы одну пару координат";
      }

      if (
        !coordinates.every(
          (point) => Array.isArray(point) && point.length === 2
        )
      ) {
        return "Каждая строка должна содержать две координаты";
      }

      if (
        !coordinates.every((point) =>
          point.every((coord) => !isNaN(coord) && typeof coord === "number")
        )
      ) {
        return "Координаты должны быть числами";
      }

      return true;
    } catch {
      return "Неверный формат координат";
    }
  };

  return (
    <Paper className="w-full h-full p-3 flex flex-col gap-10">
      <LanguageSwitcher />
      <div className="flex items-center gap-4">
        <button
          className="flex items-center gap-2"
          onClick={() => navigate(-1)}
        >
          <ArrowLeft size={20} />
          Назад
        </button>
      </div>

      <div className="flex flex-col gap-10 w-full items-end">
        <Box className="flex flex-col gap-6 w-full">
          <FormControl fullWidth required>
            <InputLabel>Выберите перевозчика</InputLabel>
            <Select
              value={editRouteData.carrier_id}
              label="Выберите перевозчика"
              onChange={(e) =>
                routeStore.setEditRouteData({
                  carrier_id: Number(e.target.value),
                  carrier:
                    carrierStore.carriers[
                      language as keyof typeof carrierStore.carriers
                    ].data?.find((c) => c.id === Number(e.target.value))
                      ?.full_name || "",
                })
              }
              disabled={
                carrierStore.carriers[
                  language as keyof typeof carrierStore.carriers
                ].data?.length === 0
              }
            >
              <MenuItem value="">Не выбрано</MenuItem>
              {carrierStore.carriers[
                language as keyof typeof carrierStore.carriers
              ].data?.map((carrier) => (
                <MenuItem key={carrier.id} value={carrier.id}>
                  {carrier.full_name}
                </MenuItem>
              ))}
            </Select>
          </FormControl>
          <TextField
            className="w-full"
            label="Номер маршрута"
            required
            value={editRouteData.route_number || ""}
            onChange={(e) =>
              routeStore.setEditRouteData({
                route_number: e.target.value,
              })
            }
          />
          <TextField
            className="w-full max-h-[300px] overflow-y-scroll -mt-5 h-full"
            multiline
            minRows={4}
            value={coordinates}
            onChange={(e) => {
              const newValue = e.target.value;
              setCoordinates(newValue);

              const validationResult = validateCoordinates(newValue);
              if (validationResult === true) {
                const lines = newValue.trim().split("\n");
                const path = lines.map((line) => {
                  const [lat, lon] = line
                    .trim()
                    .split(/[\s,]+/)
                    .map(Number);
                  return [lat, lon];
                });
                routeStore.setEditRouteData({ path });
              }
            }}
            onKeyDown={(e) => {
              if (e.key === "Enter") {
                const lines = coordinates.split("\n");
                const lastLine = lines[lines.length - 1];

                // Если мы на последней строке и она не пустая
                if (lastLine && lastLine.trim()) {
                  e.preventDefault();
                  const newValue = coordinates + "\n";
                  setCoordinates(newValue);
                }
              }
            }}
            error={validateCoordinates(coordinates) !== true}
            helperText={
              typeof validateCoordinates(coordinates) === "string"
                ? validateCoordinates(coordinates)
                : "Введите координаты в формате: широта долгота (можно использовать запятые или пробелы)"
            }
            placeholder="55.7558 37.6173
55.7539 37.6208"
          />
          <TextField
            className="w-full"
            label="Номер маршрута в Говорящем Городе"
            required
            value={editRouteData.route_sys_number || ""}
            onChange={(e) =>
              routeStore.setEditRouteData({
                route_sys_number: e.target.value,
              })
            }
          />
          <FormControl fullWidth required>
            <InputLabel>Обращение губернатора</InputLabel>
            <Select
              value={editRouteData.governor_appeal || ""}
              label="Обращение губернатора"
              onChange={(e) =>
                routeStore.setEditRouteData({
                  governor_appeal: Number(e.target.value),
                })
              }
              disabled={articlesStore.articleList.ru.data.length === 0}
            >
              <MenuItem value="">Не выбрано</MenuItem>
              {articlesStore.articleList.ru.data.map(
                (a: (typeof articlesStore.articleList.ru.data)[number]) => (
                  <MenuItem key={a.id} value={a.id}>
                    {a.heading}
                  </MenuItem>
                )
              )}
            </Select>
          </FormControl>
          <FormControl fullWidth required>
            <InputLabel>Прямой/обратный маршрут</InputLabel>
            <Select
              value={editRouteData.route_direction ? "forward" : "backward"}
              label="Прямой/обратный маршрут"
              onChange={(e) =>
                routeStore.setEditRouteData({
                  route_direction: e.target.value === "forward",
                })
              }
            >
              <MenuItem value="forward">Прямой</MenuItem>
              <MenuItem value="backward">Обратный</MenuItem>
            </Select>
          </FormControl>
          <TextField
            className="w-full"
            label="Масштаб (мин)"
            value={editRouteData.scale_min ?? ""}
            onChange={(e) =>
              routeStore.setEditRouteData({
                scale_min:
                  e.target.value === "" ? null : parseFloat(e.target.value),
              })
            }
          />
          <TextField
            className="w-full"
            label="Масштаб (макс)"
            value={editRouteData.scale_max ?? ""}
            onChange={(e) =>
              routeStore.setEditRouteData({
                scale_max:
                  e.target.value === "" ? null : parseFloat(e.target.value),
              })
            }
          />
          <TextField
            className="w-full"
            label="Поворот"
            value={editRouteData.rotate ?? ""}
            onChange={(e) =>
              routeStore.setEditRouteData({
                rotate:
                  e.target.value === "" ? null : parseFloat(e.target.value),
              })
            }
          />
          <TextField
            className="w-full"
            label="Центр. широта"
            value={editRouteData.center_latitude ?? ""}
            type="text"
            onChange={(e) =>
              routeStore.setEditRouteData({
                center_latitude: e.target.value,
              })
            }
          />
          <TextField
            className="w-full"
            label="Центр. долгота"
            value={editRouteData.center_longitude ?? ""}
            type="text"
            onChange={(e) =>
              routeStore.setEditRouteData({
                center_longitude: e.target.value,
              })
            }
          />
        </Box>

        <LinkedItems
          parentId={id || ""}
          type="edit"
          dragAllowed={true}
          fields={[
            { label: "Название", data: "name" },
            { label: "Описание", data: "description" },
          ]}
          onUpdate={() => {
            routeStore.getRoute(Number(id));
          }}
        />

        <div className="flex w-full justify-end">
          <Button
            variant="contained"
            color="primary"
            className="w-min flex gap-2 items-center"
            startIcon={<Save size={20} />}
            onClick={handleSave}
            disabled={isLoading}
          >
            Сохранить
          </Button>
        </div>
      </div>
    </Paper>
  );
});
